import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Extremely simple command line example to demonstrate how to pass credentials
 * to the GoCanvas webservice, get a listing of forms from the GoCanvas webservice
 * and get a list of submissions associated with a specific form
 * 
 * @author cmorrisette
 * 
 */
public class CommandLine {

	private static String userName;
	private static String password;
	private static String formIndex;
	private static String formGUID;
	private static String operationChoice;
	private static ArrayList formIdList = new ArrayList();
	private static String hostname = "www.gocanvas.com";
	private static String protocol = "https";

	/**
	 * Main method is a shell to get user input of what main path to go down
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.print("GoCanvas username:");
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		try {
			userName = br.readLine();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
		System.out.print("GoCanvas password:");
		try {
			password = br.readLine();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}

		System.out.println("1: Post sample reference data");
		System.out.println("2: Interact with forms");
		System.out.println("3: Interact with form GUIDs");
		System.out.println("4: Interact with CSV");
		System.out.print("Please enter a number for your choice:");
		try {
			operationChoice = br.readLine();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}

		if (operationChoice.equalsIgnoreCase("1")) {
			postReferenceDataSample();
		}
		if (operationChoice.equalsIgnoreCase("2")) {
			interactWithForms(br);
		}
		if (operationChoice.equalsIgnoreCase("3")) {
			interactWithFormGUID(br);
		}
		if (operationChoice.equalsIgnoreCase("4")) {
			interactWithCSV(br);
		}
	}

	/**
	 * Sample code to allow a user to pull back submissions associated with a
	 * form Date ranges are not currently utilized in this example but should be
	 * to limit results to a meaningfull and manageable dataset
	 * 
	 * @param br
	 */
	public static void interactWithForms(BufferedReader br) {
		System.out.println("");
		System.out.println("Form List:");

		// show the forms and get the form that the user wants to see
		// submissions for
		getFormList();
		System.out.print("Please enter the form to retrieve (0-" + (formIdList.size() - 1) + "):");
		try {
			formIndex = br.readLine().trim();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}

		// show the submissions for the specified form
		getSubmissionsById((String) formIdList.get(new Integer(formIndex).intValue()), 1);
		System.out.println("Press enter to end");
		try {
			br.readLine();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
	}

	/**
	 * An example of how to use form GUIDs instead of IDs to interact
	 * 
	 * @param br
	 */
	public static void interactWithFormGUID(BufferedReader br) {
		System.out.println("");
		System.out.println("Form GUID List:");
		printFormGUIDList();
		System.out.println("");
		System.out.print("Please enter the GUID of the form to retrieve:");
		try {
			formGUID = br.readLine().trim();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
		System.out.println("");
		getSubmissionsByGUID(formGUID, 1);
		System.out.println("Press enter to end");
		try {
			br.readLine();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
	}

	/**
	 * An example of how to post reference data into GoCanvas The XML is hardcoded
	 * and built by hand - this should of course be built using a better
	 * paradigm
	 */
	public static void postReferenceDataSample() {
		String xmlString = "<?xml version=\"1.0\" encoding=\"utf-8\"?><List Name=\"test upload\">"
				+ "<Columns><c>Column1</c><c>Column2</c><c>Column3</c></Columns>"
				+ "<Rows><r><v>Data1Column1</v><v></v><v>Data1Column3</v></r><r><v>Data2Column1</v><v>Data2Column2</v><v>Data2Column3</v></r></Rows>"
				+ "</List>";
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/reference_datas.xml?username=" + userName
					+ "&password=" + password);
			HttpURLConnection connection = (HttpURLConnection) url.openConnection();
			connection.setRequestProperty("Content-Type", "application/xml");
			connection.setRequestMethod("POST");
			connection.setDoOutput(true);
			OutputStreamWriter out = new OutputStreamWriter(connection.getOutputStream());
			out.write(xmlString);
			out.close();

			BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));

			String decodedString;

			while ((decodedString = in.readLine()) != null) {
				System.out.println(decodedString);
			}
			in.close();

		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * Methods for pulling down CSVs for a specific form version
	 * 
	 * @param br
	 */
	public static void interactWithCSV(BufferedReader br) {
		System.out.println("");
		System.out.println("Form List:");

		// show the forms and get the form that the user wants to see
		// submissions for
		getFormList();
		System.out.print("Please enter the form to retrieve (0-" + (formIdList.size() - 1) + "):");
		try {
			formIndex = br.readLine().trim();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
		System.out.println("");
		System.out.println("Form List And Submission Count:");
		// show the count of submissions that exist for each form version
		printFormListAndSubmissionCount((String) formIdList.get(new Integer(formIndex).intValue()));
		System.out.println("");
		System.out.print("Please enter the specific form version to retrieve a CSV for (0-" + (formIdList.size() - 1)
				+ "):");
		try {
			formIndex = br.readLine().trim();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
		printCSV((String) formIdList.get(new Integer(formIndex).intValue()));
		System.out.println("Press enter to end");
		try {
			br.readLine();
		} catch (IOException ioe) {
			System.out.println("IO error trying to read");
		}
	}

	/**
	 * Pull back the list of forms associated with the specified username and
	 * password and display them
	 */
	public static void getFormList() {
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/forms.xml?username=" + userName + "&password="
					+ password);
			HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
			urlCon.connect(); // Connecting
			InputStream iStream = urlCon.getInputStream();
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(iStream);
			NodeList listOfForms = doc.getElementsByTagName("Form");
			for (int i = 0; i < listOfForms.getLength(); i++) {
				String id;
				Node formNode = listOfForms.item(i);
				NamedNodeMap nodeMap = formNode.getAttributes();
				Node idNode = nodeMap.getNamedItem("Id");
				id = idNode.getNodeValue();
				formIdList.add(id);
				for (int x = 0; x < formNode.getChildNodes().getLength(); x++) {
					Node childNode = formNode.getChildNodes().item(x);
					if (childNode.getNodeName().equalsIgnoreCase("name")) {
						System.out.println("  " + (formIdList.size() - 1) + ": " + childNode.getTextContent());
					}
				}

			}
		} catch (Throwable t) {
			t.printStackTrace();
		}

	}

	/**
	 * Pull back the form names and GUIDs
	 */
	public static void printFormGUIDList() {
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/forms.xml?username=" + userName + "&password="
					+ password);
			HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
			urlCon.connect(); // Connecting
			InputStream iStream = urlCon.getInputStream();
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(iStream);
			NodeList listOfForms = doc.getElementsByTagName("Form");
			System.out.println("");
			for (int i = 0; i < listOfForms.getLength(); i++) {
				String guid;
				Node formNode = listOfForms.item(i);
				NamedNodeMap nodeMap = formNode.getAttributes();
				Node guidNode = nodeMap.getNamedItem("GUID");
				guid = guidNode.getNodeValue();
				for (int x = 0; x < formNode.getChildNodes().getLength(); x++) {
					Node childNode = formNode.getChildNodes().item(x);
					if (childNode.getNodeName().equalsIgnoreCase("name")) {
						System.out.println("  GUID:" + guid + "    Name: " + childNode.getTextContent());
					}
				}

			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * Print out the form list and submission count for specific form versions
	 * 
	 * @param formId
	 */
	public static void printFormListAndSubmissionCount(String formId) {
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/csv_meta_datas.xml?username=" + userName
					+ "&password=" + password + "&form_id=" + formId + "&begin_second=962461752&end_second=1593613752");
			HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
			urlCon.connect(); // Connecting
			InputStream iStream = urlCon.getInputStream();
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(iStream);
			NodeList listOfForms = doc.getElementsByTagName("Form");
			formIdList.clear();
			for (int i = 0; i < listOfForms.getLength(); i++) {
				String id;
				Node formNode = listOfForms.item(i);
				NamedNodeMap nodeMap = formNode.getAttributes();
				Node idNode = nodeMap.getNamedItem("Id");
				id = idNode.getNodeValue();
				String formName = null;
				String formStatus = null;
				String formVersion = null;
				String formSubmissionCount = null;
				for (int x = 0; x < formNode.getChildNodes().getLength(); x++) {
					Node childNode = formNode.getChildNodes().item(x);
					if (childNode.getNodeName().equalsIgnoreCase("name")) {
						formName = childNode.getTextContent();
					}
					if (childNode.getNodeName().equalsIgnoreCase("Status")) {
						formStatus = childNode.getTextContent();
					}
					if (childNode.getNodeName().equalsIgnoreCase("Version")) {
						formVersion = childNode.getTextContent();
					}
					if (childNode.getNodeName().equalsIgnoreCase("SubmissionCount")) {
						formSubmissionCount = childNode.getTextContent();
					}
				}

				System.out.println(new Integer(formIdList.size()).toString() + ":   ID: " + id + " Name: " + formName
						+ " Status: " + formStatus + " Version: " + formVersion + " Submission Count: "
						+ formSubmissionCount);
				formIdList.add(id);

			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * Print out the actual CSV
	 * 
	 * @param formId
	 */
	public static void printCSV(String formId) {
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/csv.xml?username=" + userName + "&password="
					+ password + "&form_id=" + formId + "&begin_second=962461752&end_second=1593613752");
			HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
			urlCon.connect(); // Connecting
			BufferedReader in = new BufferedReader(new InputStreamReader(urlCon.getInputStream()));
			String line = null;
			while ((line = in.readLine()) != null) {
				System.out.println(line);
			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * Pull back the submissions XML associated the user and form
	 * 
	 * @param formId
	 */
	public static void getSubmissionsById(String formId, int page_number) {
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/submissions.xml?username=" + userName
					+ "&password=" + password + "&form_id=" + formId + "&page=" + page_number);
			HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
			urlCon.connect(); // Connecting
			InputStream iStream = urlCon.getInputStream();
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(iStream);
			String num_pages = doc.getElementsByTagName("TotalPages").item(0).getTextContent();
			String current_page = doc.getElementsByTagName("CurrentPage").item(0).getTextContent();
			System.out.println("Total Pages " + num_pages);
			System.out.println("Current Page " + current_page);
			NodeList listOfSubmissions = doc.getElementsByTagName("Submission");
			System.out.println("");
			for (int i = 0; i < listOfSubmissions.getLength(); i++) {
				Node submissionNode = listOfSubmissions.item(i);
				processSubmission(submissionNode);
			}
			if (listOfSubmissions.getLength() == 0) {
				System.out.println("No submissions found");
			}
			if(Integer.parseInt(num_pages) > page_number)
			{
				getSubmissionsById(formId, (page_number +1));
			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * Pull back the submissions XML associated the user and form
	 * 
	 * @param formId
	 */
	public static void getSubmissionsByGUID(String formGUID, int page_number) {
		try {
			URL url = new URL(protocol + "://" + hostname + "/apiv2/submissions.xml?username=" + userName
					+ "&password=" + password + "&form_guid=" + formGUID + "&page=" + page_number);
			HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
			urlCon.connect(); // Connecting
			InputStream iStream = urlCon.getInputStream();
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(iStream);
			String num_pages = doc.getElementsByTagName("TotalPages").item(0).getTextContent();
			String current_page = doc.getElementsByTagName("CurrentPage").item(0).getTextContent();
			System.out.println("Total Pages " + num_pages);
			System.out.println("Current Page " + current_page);
			NodeList listOfSubmissions = doc.getElementsByTagName("Submission");
			System.out.println("");
			for (int i = 0; i < listOfSubmissions.getLength(); i++) {
				Node submissionNode = listOfSubmissions.item(i);
				processSubmission(submissionNode);
			}
			if (listOfSubmissions.getLength() == 0) {
				System.out.println("No submissions found");
			}
			if(Integer.parseInt(num_pages) > page_number)
			{
				getSubmissionsByGUID(formGUID, (page_number +1));
			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * Process and individual submission node (there can be multiple submissions
	 * in the XML document)
	 * 
	 * @param submissionNode
	 */
	public static void processSubmission(Node submissionNode) {
		System.out.println("");
		System.out.println("--------- Begin Submission -----------");
		for (int i = 0; i < submissionNode.getChildNodes().getLength(); i++) {
			Node childNode = submissionNode.getChildNodes().item(i);
			if (childNode.getNodeName().equalsIgnoreCase("ResponseID")) {
				System.out.println("ResponseID: " + childNode.getTextContent());
			}
			if (childNode.getNodeName().equalsIgnoreCase("Sections")) {
				for (int x = 0; x < childNode.getChildNodes().getLength(); x++) {
					Node sectionNode = childNode.getChildNodes().item(x);
					if (sectionNode.getNodeName().equalsIgnoreCase("Section")) {
						processSection(sectionNode);
					}
				}
			}
		}
		System.out.println("--------- End Submission -----------");
	}

	/**
	 * Process a section, this can occur directly within a submission or nested
	 * within a response group (when there is a repeating list of data)
	 * 
	 * @param sectionNode
	 */
	public static void processSection(Node sectionNode) {
		for (int i = 0; i < sectionNode.getChildNodes().getLength(); i++) {
			Node childNode = sectionNode.getChildNodes().item(i);
			if (childNode.getNodeName().equalsIgnoreCase("Name")) {
				System.out.println("Section: " + childNode.getTextContent().trim());
			}
			if (childNode.getNodeName().equalsIgnoreCase("Screens")) {
				for (int x = 0; x < childNode.getChildNodes().getLength(); x++) {
					Node screenNode = childNode.getChildNodes().item(x);
					if (screenNode.getNodeName().equalsIgnoreCase("Screen")) {
						processscreen(screenNode);
					}
				}
			}
		}
	}

	/**
	 * Process a screen which contains with responses or response groups
	 * 
	 * @param screenNode
	 */
	public static void processscreen(Node screenNode) {
		for (int i = 0; i < screenNode.getChildNodes().getLength(); i++) {
			Node responsesNode = screenNode.getChildNodes().item(i);
			if (responsesNode.getNodeName().equalsIgnoreCase("Responses")) {
				processResponses(responsesNode);
			}
			if (responsesNode.getNodeName().equalsIgnoreCase("ResponseGroups")) {
				processResponseGroup(responsesNode);
			}
		}
	}

	/**
	 * Process an individual response
	 * 
	 * @param responsesNode
	 */
	public static void processResponses(Node responsesNode) {
		for (int y = 0; y < responsesNode.getChildNodes().getLength(); y++) {
			Node childNode = responsesNode.getChildNodes().item(y);
			if (childNode.getNodeName().equalsIgnoreCase("Response")) {
				processResponse(childNode);
			}

		}
	}

	/**
	 * Process a response group, which contains a single answer associated with
	 * a section
	 * 
	 * @param responseGroupsNode
	 */
	public static void processResponseGroup(Node responseGroupsNode) {
		for (int i = 0; i < responseGroupsNode.getChildNodes().getLength(); i++) {
			Node responseGroupNode = responseGroupsNode.getChildNodes().item(i);
			if (responseGroupNode.getNodeName().equalsIgnoreCase("ResponseGroup")) {
				for (int x = 0; x < responseGroupNode.getChildNodes().getLength(); x++) {
					Node childNode = responseGroupNode.getChildNodes().item(x);
					if (childNode.getNodeName().equalsIgnoreCase("Response")) {
						processResponse(childNode);
					}
					if (childNode.getNodeName().equalsIgnoreCase("Section")) {
						processSection(childNode);
					}
				}
			}
		}
	}

	/**
	 * Output the data associated with the response
	 * 
	 * @param responseNode
	 */
	public static void processResponse(Node responseNode) {
		String label = "";
		String value = "";
		String type = "";
		for (int x = 0; x < responseNode.getChildNodes().getLength(); x++) {
			Node responseData = responseNode.getChildNodes().item(x);
			if (responseData.getNodeName().equalsIgnoreCase("Label")) {
				label = responseData.getTextContent().trim();
			}
			if (responseData.getNodeName().equalsIgnoreCase("Value")) {
				value = responseData.getTextContent().trim();
			}
			if (responseData.getNodeName().equalsIgnoreCase("Type")) {
				type = responseData.getTextContent().trim();
			}
		}
		System.out.println("(Response) " + label + ": " + value + "(" + type + ")");
	}

}
